unit TransLT.Obj;

interface

uses
  Winapi.Windows, System.SysUtils, Vcl.Dialogs, Vcl.Graphics, TransLT.API;

//----------------------------------------------------------------------------------------------------------------------
//Useful for html colors
function ColorToStringHtml(const OpacityProcent: byte; const AColor: TColor): string; overload;
function ColorToStringHtml(const OpacityProcent, Red, Green, Blue: byte): string; overload;
procedure StringHtmlToColor(const SColor: string; var OpacityProcent: byte; var AColor: TColor); overload;
procedure StringHtmlToColor(const SColor: string; var OpacityProcent, Red, Green, Blue: byte); overload;

//----------------------------------------------------------------------------------------------------------------------
//Functions of language settings for error messages
procedure SetErrMsgFile(const Value: string);
function GetErrMsgFile: string;

//----------------------------------------------------------------------------------------------------------------------
//Functions about model properties read directly from mtr, mtrs or xml files (model or models files)
function GetModelFileIsCompatible(const FileName: string): boolean;
function GetCountModelsFromFile(const FileName: string): integer;
function GetModelHasCooOperations(const FileName: string; const ModelIndex: integer): boolean;
function GetModelIsReversible(const FileName: string; const ModelIndex: integer): boolean;
function GetModelIsCompatToDrawInGE(const FileName: string; const ModelIndex: integer): boolean;
function GetModelDescription(const FileName: string; const ModelIndex: integer;
  var ModelDescription: TModelDescription): boolean;

//----------------------------------------------------------------------------------------------------------------------
//Useful for strings
procedure StrWCopy(Dest: PWideChar; const Source: PChar; MaxLen: Cardinal);

//----------------------------------------------------------------------------------------------------------------------
//Final cleanup if temporary files have been created at the end of the work session
procedure ClearTempFiles;

//----------------------------------------------------------------------------------------------------------------------
type
  //Class for universal coordinate transformations of points by using a TransLT
  //transformation model.
  TTransformPoints = class
  private
    FModelFileName   : string;
    FPlainUnits      : TPlainUnits;
    FUseReversed     : boolean;
    FGridFilesPath   : string;
    FChDeg           : WideChar;
    FChMin           : WideChar;
    FChSec           : WideChar;
    FChCsv           : WideChar;
    FUseDecDeg       : boolean;
    FUseNSEW         : boolean;
    FDecNoXYZ        : integer;
    FDecNoNEH        : integer;
    FDecNoSec        : integer;
    FDecNoGeoHe      : integer;
    FInvertAxes12    : boolean;
    FModelDescription: TModelDescription;
    FModelIsLoaded   : boolean;
    FIsTimeDependent : boolean;
    FIsUseTimeEpochT0: boolean;
    function GetCountCooPoints: integer;
    function GetAngleAspect: TAngleAspect;
    function GetEpochT0: extended;
    procedure SetEpochT0(const Value: extended);
    function GetEpochT: extended;
    procedure SetEpochT(const Value: extended);
  public
    constructor Create(const ModelFileName: string; const ModelIndex: integer; const PlainUnits: TPlainUnits;
      const UseReversed: boolean; const GridFilesPath: string);
    destructor Destroy; override;

    procedure SetModelFile(const ModelFileName: string; const ModelIndex: integer; const PlainUnits: TPlainUnits;
      const UseReversed: boolean; const GridFilesPath: string);
    procedure AddCooPoint(const PntName: string; const XorEorPhi, YorNorLam, ZorHorHe: extended; const PntCode: string);
    procedure SetCooPoint(const Index: integer; const PntName: string; const XorEorPhi, YorNorLam, ZorHorHe: extended;
      const PntCode: string);
    function GetCooPoint(const Index: integer; var PntName: string; var XorEorPhi, YorNorLam, ZorHorHe: extended;
      var PntCode: string): boolean;
    function GetCooPointFormated(const Index: integer; var PntName, StrXorEorPhi, StrYorNorLam, StrZorHorHe,
      PntCode: string): boolean;
    procedure DeleteCooPoint(const Index: integer);
    procedure DeleteAllCooPoints;
    function LoadCooPointsFromFile(const FileName: string): boolean;
    function SaveCooPointsToFile(const FileName: string; const FixCoo: boolean;
      const EncodingType: TEncodingType): boolean;
    function ApplyTransformation(ProgressProc: TProgressProc; const Silent: boolean = False): boolean;

    //Properties about model
    property ModelFileName: string read FModelFileName;
    property PlainUnits: TPlainUnits read FPlainUnits;
    property UseReversed: boolean read FUseReversed;
    property GridFilesPath: string read FGridFilesPath;
    property ModelDescription: TModelDescription read FModelDescription;
    property ModelIsLoaded: boolean read FModelIsLoaded;
    property IsTimeDependent: boolean read FIsTimeDependent;
    property IsUseTimeEpochT0: boolean read FIsUseTimeEpochT0;
    property EpochT0: extended read GetEpochT0 write SetEpochT0;
    property EpochT: extended read GetEpochT write SetEpochT;

    //Properties about points
    property CountCooPoints: integer read GetCountCooPoints;

    //Values used only at read and write coordinates from file
    property ChDeg: WideChar read FChDeg write FChDeg;
    property ChMin: WideChar read FChMin write FChMin;
    property ChSec: WideChar read FChSec write FChSec;
    property ChCsv: WideChar read FChCsv write FChCsv;
    property UseDecDeg: boolean read FUseDecDeg write FUseDecDeg;
    property UseNSEW: boolean read FUseNSEW write FUseNSEW;
    property DecNoXYZ: integer read FDecNoXYZ write FDecNoXYZ;
    property DecNoNEH: integer read FDecNoNEH write FDecNoNEH;
    property DecNoSec: integer read FDecNoSec write FDecNoSec;
    property DecNoGeoHe: integer read FDecNoGeoHe write FDecNoGeoHe;
    property InvertAxes12: boolean read FInvertAxes12 write FInvertAxes12;
  end;

  //Class for universal coordinate transformations of points of entities by
  //using a TransLT transformation model and for drawing entities in Google
  //Earth. Entities are drawn only in 2D dimension and are represented in
  //Google Earth at the ground level.
  TTransformEntitiesGE = class
  private
    FModelFileName   : string;
    FPlainUnits      : TPlainUnits;
    FAreaUnits       : TAreaUnits;
    FLengthUnits     : TPlainUnits;
    FUseReversed     : boolean;
    FGridFilesPath   : string;
    FChDeg           : WideChar;
    FChMin           : WideChar;
    FChSec           : WideChar;
    FChCsv           : WideChar;
    FUseDecDeg       : boolean;
    FUseNSEW         : boolean;
    FDecNoNEH        : integer;
    FDecNoSec        : integer;
    FInvertAxes12    : boolean;
    FDecNoArea       : integer;
    FDecNoLength     : integer;
    FProgName        : string;
    FProgVer         : string;
    FWebPage         : string;
    FPlaceName       : string;
    FModelDescription: TModelDescription;
    FGEDrawOptions   : TGEDrawOptions;
    FModelIsLoaded   : boolean;
    FIsTimeDependent : boolean;
    FIsUseTimeEpochT0: boolean;
    procedure SetPlaceName(const Value: string);
    function GetAngleAspect: TAngleAspect;
    function GetEpochT0: extended;
    procedure SetEpochT0(const Value: extended);
    function GetEpochT: extended;
    procedure SetEpochT(const Value: extended);
    procedure SetGEDrawOptions(const Value: TGEDrawOptions);
  public
    constructor Create(const ModelFileName: string; const ModelIndex: integer; const PlainUnits: TPlainUnits;
      const UseReversed: boolean; const GridFilesPath: string);
    destructor Destroy; override;

    procedure SetModelFile(const ModelFileName: string; const ModelIndex: integer; const PlainUnits: TPlainUnits;
      const UseReversed: boolean; const GridFilesPath: string);

    procedure AddEntity(const GEEntityProperties: TGEEntityProperties);
    procedure SetEntity(const EntIndex: integer; const GEEntityProperties: TGEEntityProperties);
    procedure DeleteEntity(const EntIndex: integer);
    function GetEntity(const EntIndex: integer; var GEEntityProperties: TGEEntityProperties): boolean;
    procedure ClearEntities;
    function GetCountEntites: integer;
    procedure AddPointToEntity(const EntIndex: integer; const PntName: string; const EorPhi, NorLam: extended);
    procedure SetPointToEntity(const EntIndex, PntIndex: integer; const PntName: string;
      const EorPhi, NorLam: extended);
    function GetPointFromEntity(const EntIndex, PntIndex: integer; var PntName: string;
      var EorPhi, NorLam: extended): boolean;
    function GetPointFromEntityFormated(const EntIndex, PntIndex: integer; var PntName, StrEorPhi, StrNorLam: string;
      const CoordIsGeog: boolean): boolean;
    function GetCountPointsOfEntity(const EntIndex: integer): integer;
    procedure DeleteAllPointsOfEntity(const EntIndex: integer);
    procedure CalculateAreaAndLengthToEntity(const EntIndex: integer);
    function LoadEntitiesFromFile(const FileName: string): boolean;
    function SaveEntitiesToFile(const FileName: string; const EncodingType: TEncodingType): boolean;
    function DrawEntitiesToGoogleEarth(const TempFileName: string = ''): boolean;
    function SaveZoomToFile(const FileName, FolderName: string; const PhiMin, LamMin, PhiMax, LamMax: extended)
      : boolean;
    function DrawZoomToGoogleEarth(const PhiMin, LamMin, PhiMax, LamMax: extended; const FolderName: string = '';
      const TempFileName: string = ''): boolean;
    function TransformPointsOfAllEntities(ProgressProc: TProgressProc): boolean;

    //Properties about model
    property ModelFileName: string read FModelFileName;
    property PlainUnits: TPlainUnits read FPlainUnits;
    property UseReversed: boolean read FUseReversed;
    property GridFilesPath: string read FGridFilesPath;
    property ModelDescription: TModelDescription read FModelDescription;
    property ModelIsLoaded: boolean read FModelIsLoaded;
    property IsTimeDependent: boolean read FIsTimeDependent;
    property IsUseTimeEpochT0: boolean read FIsUseTimeEpochT0;
    property EpochT0: extended read GetEpochT0 write SetEpochT0;
    property EpochT: extended read GetEpochT write SetEpochT;

    //Values used only at read and write coordinates from file
    property ChDeg: WideChar read FChDeg write FChDeg;
    property ChMin: WideChar read FChMin write FChMin;
    property ChSec: WideChar read FChSec write FChSec;
    property ChCsv: WideChar read FChCsv write FChCsv;
    property UseDecDeg: boolean read FUseDecDeg write FUseDecDeg;
    property UseNSEW: boolean read FUseNSEW write FUseNSEW;
    property DecNoNEH: integer read FDecNoNEH write FDecNoNEH;
    property DecNoSec: integer read FDecNoSec write FDecNoSec;
    property InvertAxes12: boolean read FInvertAxes12 write FInvertAxes12;
    property DecNoArea: integer read FDecNoArea write FDecNoArea;
    property DecNoLength: integer read FDecNoLength write FDecNoLength;
    property AreaUnits: TAreaUnits read FAreaUnits write FAreaUnits;
    property LengthUnits: TPlainUnits read FLengthUnits write FLengthUnits;
    property ProgName: string read FProgName write FProgName;
    property ProgVer: string read FProgVer write FProgVer;
    property WebPage: string read FWebPage write FWebPage;
    property PlaceName: string read FPlaceName write SetPlaceName;
    property GEDrawOptions: TGEDrawOptions read FGEDrawOptions write SetGEDrawOptions;
  end;

var
  GEEntLangWords : TGEEntLangWords;
  GEKmlLangWords : TGEKmlLangWords;
  StrFoldPolygons: string;
  StrFoldPaths   : string;
  StrFoldGroupPts: string;
  StrCooOpForErr : string;
  TempFilesIsUsed: boolean = False;

const
{$IFDEF UNICODE}
  CH_PHI    = WideString(#$03C6); //Greek Small Letter Phi
  CH_LAMBDA = WideString(#$03BB); //Greek Small Letter Lambda
{$ELSE}
  CH_PHI    = 'phi';
  CH_LAMBDA = 'lambda';
{$ENDIF}

implementation

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
procedure ClearTempFiles;
begin
  if TempFilesIsUsed then
    ClearTempFilesGE;

  TempFilesIsUsed:= False;
end;

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
procedure StrWCopy(Dest: PWideChar; const Source: PChar; MaxLen: Cardinal);
var
  i, Len: Cardinal;
begin
  Len:= StrLen(Source);
  if Len > MaxLen then
    Len:= MaxLen;

  if Len > 0 then
    for i:= 0 to Len - 1 do
      begin
        Dest[i]:= WideChar(Source[i]);
      end;

  Dest[Len]:= #0;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure ChangeToAnsiPhiLambdaAndArrows(PText: PWideChar; const MaxLen: Cardinal);
var
  S     : WideString;
  i, Len: Cardinal;
begin
  Len:= Length(PText);
  S  := '';
  if Len > 0 then
    begin
      for i:= 0 to Len - 1 do
        begin
          if WideChar(PText[i]) = #$03C6 then //Greek Small Letter Phi
            S:= S + WideString(CH_PHI)
          else if WideChar(PText[i]) = #$03BB then //Greek Small Letter Lambda
            S:= S + WideString(CH_LAMBDA)

          else if WideChar(PText[i]) = #$2190 then //Leftwards Arrow
            S:= S + WideString('<-')
          else if WideChar(PText[i]) = #$2192 then //Rightwards Arrow
            S:= S + WideString('->')
          else if WideChar(PText[i]) = #$2194 then //Left Right Arrow
            S:= S + WideString('<->')
          else if WideChar(PText[i]) = #$21C4 then //Rightwards Arrow Over Leftwards Arrow
            S:= S + WideString('<->')
          else
            S:= S + WideChar(PText[i]);
        end;

      Len:= Length(S);
      if Len > MaxLen then
        Len:= MaxLen;
      CopyMemory(PText, PWideChar(S), Len * SizeOf(WideChar));
      PText[Len]:= #0;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure ChangeCharsOfModelDescription(var ModelDescription: TModelDescription);
begin
  with ModelDescription do
    begin
      ChangeToAnsiPhiLambdaAndArrows(ModelName, MAX_LEN_NAMES);
      ChangeToAnsiPhiLambdaAndArrows(TransDescription, MAX_LEN_NAMES);
      ChangeToAnsiPhiLambdaAndArrows(CooSysIniName, MAX_LEN_NAMES);
      ChangeToAnsiPhiLambdaAndArrows(CooSysFinName, MAX_LEN_NAMES);
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
{$IFNDEF UNICODE}

function WideStringToStringEx(lpWideChar: PWideChar): string;
var
  L       : Integer;
  CodePage: Word;
begin
  CodePage:= GetACP;
  L       := WideCharToMultiByte(CodePage, 0, lpWideChar, -1, nil, 0, nil, nil);
  SetLength(Result, L - 1);
  WideCharToMultiByte(CodePage, 0, lpWideChar, -1, PChar(Result), L - 1, nil, nil);
end;
{$ENDIF}

//----------------------------------------------------------------------------------------------------------------------
procedure ShowMessageError(const ErrNo: integer; lpErrTxt: PWideChar; nMaxCount: integer);
var
  ErrTxt: string;
begin
{$IFNDEF UNICODE}
  ChangeToAnsiPhiLambdaAndArrows(lpErrTxt, 1024);
  ErrTxt:= WideStringToStringEx(lpErrTxt);
{$ELSE}
  SetString(ErrTxt, lpErrTxt, nMaxCount);
{$ENDIF}
{$IFDEF TOPOLT}
  MessageDlgPosHelp(ErrTxt, mtError, [mbOK], 0, -1, -1, '');
{$ELSE}
  MessageDlgPosHelp('Error Number: ' + IntToStr(ErrNo) + ' (line number from .err text file)' + sLineBreak +
    'Text Message: ' + sLineBreak + '________________________________________________________' + sLineBreak + sLineBreak
    + ErrTxt, mtError, [mbOK], 0, -1, -1, '');
{$ENDIF}
end;

//----------------------------------------------------------------------------------------------------------------------
//Html colors
//----------------------------------------------------------------------------------------------------------------------
function GetColorFromStringHtml(const SColor: string): TColor;
begin
  Try
    Result:= StringToColor('$' + Copy(SColor, 3, 6));
  Except
    Result:= clWhite;
  End;
end;

//----------------------------------------------------------------------------------------------------------------------
function GetOpacityProcentFromStringHtml(const SColor: string): byte;
var
  Opacity: byte;
begin
  Try
    Opacity:= StrToInt('$' + Copy(SColor, 1, 2));
    Result := Round(100 * (Opacity / 255));
  Except
    Result:= 100;
  End;
end;

//----------------------------------------------------------------------------------------------------------------------
function ColorToStringHtml(const OpacityProcent: byte; const AColor: TColor): string;
var
  Opacity: byte;
begin
  Opacity:= Trunc(255 * OpacityProcent / 100);
  FmtStr(Result, '%s%.2x', ['', Opacity]);
  FmtStr(Result, '%s%.6x', [Result, AColor]);
  Result:= LowerCase(Result);
end;

//----------------------------------------------------------------------------------------------------------------------
function ColorToStringHtml(const OpacityProcent, Red, Green, Blue: byte): string;
begin
  Result:= ColorToStringHtml(OpacityProcent, RGB(Red, Green, Blue));
end;

//----------------------------------------------------------------------------------------------------------------------
procedure StringHtmlToColor(const SColor: string; var OpacityProcent: byte; var AColor: TColor);
begin
  OpacityProcent:= GetOpacityProcentFromStringHtml(SColor);
  AColor        := GetColorFromStringHtml(SColor);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure StringHtmlToColor(const SColor: string; var OpacityProcent, Red, Green, Blue: byte);
var
  AColor: TColor;
begin
  StringHtmlToColor(SColor, OpacityProcent, AColor);

  Red  := GetRValue(AColor);
  Green:= GetGValue(AColor);
  Blue := GetBValue(AColor);
end;

//----------------------------------------------------------------------------------------------------------------------
//Settings for error messages
//----------------------------------------------------------------------------------------------------------------------
procedure SetErrMsgFile(const Value: string);
begin
  SetErrMsgFileName(PWideChar(WideString(Value)));
end;

//----------------------------------------------------------------------------------------------------------------------
function GetErrMsgFile: string;
var
  lpFileName: PWideChar;
  nMaxCount : integer;
begin
  GetErrMsgFileName(lpFileName, nMaxCount);
  SetString(Result, lpFileName, nMaxCount);
end;

//----------------------------------------------------------------------------------------------------------------------
//Model properties from file
//----------------------------------------------------------------------------------------------------------------------
function GetModelFileIsCompatible(const FileName: string): boolean;
begin
  Result:= ModelFileIsCompatible(PWideChar(WideString(FileName)));
end;

//----------------------------------------------------------------------------------------------------------------------
function GetCountModelsFromFile(const FileName: string): integer;
begin
  Result:= GetCountModels(PWideChar(WideString(FileName)));
end;

//----------------------------------------------------------------------------------------------------------------------
function GetModelHasCooOperations(const FileName: string; const ModelIndex: integer): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  Result:= ModelHasCooOperations(PWideChar(WideString(FileName)), ModelIndex, ErrNo, lpErrTxt, nMaxCount);
  if ErrNo > 0 then
    ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
end;

//----------------------------------------------------------------------------------------------------------------------
function GetModelIsReversible(const FileName: string; const ModelIndex: integer): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  Result:= ModelIsReversible(PWideChar(WideString(FileName)), ModelIndex, ErrNo, lpErrTxt, nMaxCount);
  if ErrNo > 0 then
    ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
end;

//----------------------------------------------------------------------------------------------------------------------
function GetModelIsCompatToDrawInGE(const FileName: string; const ModelIndex: integer): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  Result:= ModelIsCompatibleToDrawInGoogleEarth(PWideChar(WideString(FileName)), ModelIndex, ErrNo, lpErrTxt,
    nMaxCount);
  if ErrNo > 0 then
    ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
end;

//----------------------------------------------------------------------------------------------------------------------
function GetModelDescription(const FileName: string; const ModelIndex: integer;
  var ModelDescription: TModelDescription): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  Result:= GetModelDescriptionFromFile(PWideChar(WideString(FileName)), ModelIndex, ModelDescription, ErrNo, lpErrTxt,
    nMaxCount);
  if ErrNo > 0 then
    ShowMessageError(ErrNo, lpErrTxt, nMaxCount)
  else
{$IFNDEF UNICODE}
    ChangeCharsOfModelDescription(ModelDescription);
{$ENDIF}
end;

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
{ TTransformPoints }
//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
constructor TTransformPoints.Create(const ModelFileName: string; const ModelIndex: integer;
  const PlainUnits: TPlainUnits; const UseReversed: boolean; const GridFilesPath: string);
begin
  FChDeg:= '';
  FChMin:= '''';
  FChSec:= '"';
{$IFDEF CONDITIONALEXPRESSIONS}
{$IF CompilerVersion >= 20.0}
  FChCsv:= WideChar(FormatSettings.ListSeparator);
{$ELSE}
  FChCsv:= WideChar(SysUtils.ListSeparator);
{$IFEND}
{$ELSE}
  FChCsv:= WideChar(SysUtils.ListSeparator);
{$ENDIF}
  FUseDecDeg       := False;
  FUseNSEW         := True;
  FDecNoXYZ        := 4;
  FDecNoNEH        := 3;
  FDecNoSec        := 6;
  FDecNoGeoHe      := 3;
  FInvertAxes12    := False;
  FIsTimeDependent := False;
  FIsUseTimeEpochT0:= False;

  SetModelFile(ModelFileName, ModelIndex, PlainUnits, UseReversed, GridFilesPath);
end;

//----------------------------------------------------------------------------------------------------------------------
destructor TTransformPoints.Destroy;
begin
  DoneTrPntModel;
  inherited;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformPoints.SetModelFile(const ModelFileName: string; const ModelIndex: integer;
  const PlainUnits: TPlainUnits; const UseReversed: boolean; const GridFilesPath: string);
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  FillChar(FModelDescription, SizeOf(TModelDescription), 0);

  FModelFileName:= '';
  FPlainUnits   := PlainUnits;
  FUseReversed  := UseReversed;
  FGridFilesPath:= '';
  FModelIsLoaded:= False;

  if InitTrPntModel(PWideChar(WideString(ModelFileName)), PWideChar(WideString(GridFilesPath)), ModelIndex, FPlainUnits,
    UseReversed, ErrNo, lpErrTxt, nMaxCount) then
    begin
      if GetTrPntModelDescription(FModelDescription) then
        begin
          FModelIsLoaded   := True;
          FModelFileName   := ModelFileName;
          FGridFilesPath   := GridFilesPath;
          FIsTimeDependent := FModelDescription.IsTimeDependent;
          FIsUseTimeEpochT0:= FModelDescription.IsUseTimeEpochT0;

{$IFNDEF UNICODE}
          ChangeCharsOfModelDescription(FModelDescription);
{$ENDIF}
        end;
    end
  else
    begin
      if ErrNo > 0 then
        ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformPoints.SetEpochT0(const Value: extended);
begin
  SetTrPntEpochT0(Value);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.GetEpochT0: extended;
begin
  if not GetTrPntEpochT0(Result) then
    Result:= 0.0;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformPoints.SetEpochT(const Value: extended);
begin
  SetTrPntEpochT(Value);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.GetEpochT: extended;
begin
  if not GetTrPntEpochT(Result) then
    Result:= 0.0;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformPoints.AddCooPoint(const PntName: string; const XorEorPhi, YorNorLam, ZorHorHe: extended;
  const PntCode: string);
begin
  AddPoint(PWideChar(WideString(PntName)), XorEorPhi, YorNorLam, ZorHorHe, PWideChar(WideString(PntCode)));
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformPoints.SetCooPoint(const Index: integer; const PntName: string;
  const XorEorPhi, YorNorLam, ZorHorHe: extended; const PntCode: string);
begin
  SetPoint(Index, PWideChar(WideString(PntName)), XorEorPhi, YorNorLam, ZorHorHe, PWideChar(WideString(PntCode)));
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.GetCooPoint(const Index: integer; var PntName: string;
  var XorEorPhi, YorNorLam, ZorHorHe: extended; var PntCode: string): boolean;
var
  lpPntName, lpPntCode              : PWideChar;
  nMaxPntNameCount, nMaxPntCodeCount: integer;
begin
  Result:= GetPoint(Index, lpPntName, nMaxPntNameCount, XorEorPhi, YorNorLam, ZorHorHe, lpPntCode, nMaxPntCodeCount);
  SetString(PntName, lpPntName, nMaxPntNameCount);
  SetString(PntCode, lpPntCode, nMaxPntCodeCount);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.GetCooPointFormated(const Index: integer; var PntName, StrXorEorPhi, StrYorNorLam,
  StrZorHorHe, PntCode: string): boolean;
var
  lpPntName, lpXorEorPhi, lpYorNorLam, lpZorHorHe, lpPntCode: PWideChar;
  nMaxPntNameCount, nMaxXorEorPhiCount, nMaxYorNorLamCount, nMaxZorHorHeCount, nMaxPntCodeCount: integer;
begin
  Result:= GetPointFormated(Index, lpPntName, nMaxPntNameCount, lpXorEorPhi, nMaxXorEorPhiCount, lpYorNorLam,
    nMaxYorNorLamCount, lpZorHorHe, nMaxZorHorHeCount, lpPntCode, nMaxPntCodeCount, GetAngleAspect, FDecNoXYZ,
    FDecNoNEH, FDecNoGeoHe);
  SetString(PntName, lpPntName, nMaxPntNameCount);
  SetString(StrXorEorPhi, lpXorEorPhi, nMaxXorEorPhiCount);
  SetString(StrYorNorLam, lpYorNorLam, nMaxYorNorLamCount);
  SetString(StrZorHorHe, lpZorHorHe, nMaxZorHorHeCount);
  SetString(PntCode, lpPntCode, nMaxPntCodeCount);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformPoints.DeleteCooPoint(const Index: integer);
begin
  DeletePoint(Index);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformPoints.DeleteAllCooPoints;
begin
  DeleteAllPoints;
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.GetCountCooPoints: integer;
begin
  Result:= GetCountPoints;
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.GetAngleAspect: TAngleAspect;
begin
  with Result do
    begin
      UseDecDeg:= FUseDecDeg;
      UseNSEW  := FUseNSEW;
      ChDeg    := FChDeg;
      ChMin    := FChMin;
      ChSec    := FChSec;
      DecNoSec := FDecNoSec;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.LoadCooPointsFromFile(const FileName: string): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  Result:= LoadPointsFromFile(PWideChar(WideString(FileName)), FInvertAxes12, GetAngleAspect, FPlainUnits, FChCsv,
    ErrNo, lpErrTxt, nMaxCount);
  if not Result then
    begin
      if ErrNo > 0 then
        ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.SaveCooPointsToFile(const FileName: string; const FixCoo: boolean;
  const EncodingType: TEncodingType): boolean;
begin
  Result:= SavePointsToFile(PWideChar(WideString(FileName)), FixCoo, FInvertAxes12, GetAngleAspect, FPlainUnits, FChCsv,
    FDecNoXYZ, FDecNoNEH, FDecNoGeoHe, EncodingType);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformPoints.ApplyTransformation(ProgressProc: TProgressProc; const Silent: boolean = False): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  Result:= TransformPoints(ProgressProc, PWideChar(WideString(StrCooOpForErr)), ErrNo, lpErrTxt, nMaxCount);
  if (not Result) and (ErrNo > 0) and not Silent then
    ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
end;

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
{ TTransformEntitiesGE }
//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
constructor TTransformEntitiesGE.Create(const ModelFileName: string; const ModelIndex: integer;
  const PlainUnits: TPlainUnits; const UseReversed: boolean; const GridFilesPath: string);
begin
  FChDeg:= '';
  FChMin:= '''';
  FChSec:= '"';
{$IFDEF CONDITIONALEXPRESSIONS}
{$IF CompilerVersion >= 20.0}
  FChCsv:= WideChar(FormatSettings.ListSeparator);
{$ELSE}
  FChCsv:= WideChar(SysUtils.ListSeparator);
{$IFEND}
{$ELSE}
  FChCsv:= WideChar(SysUtils.ListSeparator);
{$ENDIF}
  FUseDecDeg       := False;
  FUseNSEW         := True;
  FDecNoNEH        := 3;
  FDecNoSec        := 6;
  FInvertAxes12    := False;
  FIsTimeDependent := False;
  FIsUseTimeEpochT0:= False;

  FDecNoArea  := 2;
  FDecNoLength:= 2;
  FAreaUnits  := auSqMeter;
  FLengthUnits:= puMeter;
  FProgName   := 'My Application';
  FProgVer    := '1.0';
  FWebPage    := 'http://www.topolt.com';
  FPlaceName  := MY_PLACE_INI_TEXT;

  with FGEDrawOptions do
    begin
      StrWCopy(PlgLabelsHtmlColor, PChar(ColorToStringHtml(100, 255, 170, 0)), MAX_LEN_HTML_COLOR);
      StrWCopy(PlgLinesHtmlColor, PChar(ColorToStringHtml(100, clYellow)), MAX_LEN_HTML_COLOR);
      PlgDrawLabels   := True;
      PlgAddAreaToName:= False;
      StrWCopy(PathLabelsHtmlColor, PChar('ffb88774'), MAX_LEN_HTML_COLOR);
      PathDrawLabels     := True;
      PathAddLengthToName:= True;
      PathIdxSymbIcon    := 0;
      StrWCopy(PtsLabelsHtmlColor, PChar('ffffffff'), MAX_LEN_HTML_COLOR);
      AddDescrToName    := True;
      LabelsDistToViewKm:= 10.0; //km
      StrWCopy(FoldPlg, PChar(StrFoldPolygons), MAX_LEN_FOLDER_NAME);
      StrWCopy(FoldPath, PChar(StrFoldPaths), MAX_LEN_FOLDER_NAME);
      StrWCopy(FoldGrpPts, PChar(StrFoldGroupPts), MAX_LEN_FOLDER_NAME);
      CorPhi:= 0.0; //Be careful when changing
      CorLam:= 0.0; //these values.
    end;

  SetModelFile(ModelFileName, ModelIndex, PlainUnits, UseReversed, GridFilesPath);
end;

//----------------------------------------------------------------------------------------------------------------------
destructor TTransformEntitiesGE.Destroy;
begin
  DoneTrGEModel;
  inherited;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.SetPlaceName(const Value: string);
begin
  FPlaceName:= Trim(Value);
  if FPlaceName = '' then
    FPlaceName:= MY_PLACE_INI_TEXT;
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.GetAngleAspect: TAngleAspect;
begin
  with Result do
    begin
      UseDecDeg:= FUseDecDeg;
      UseNSEW  := FUseNSEW;
      ChDeg    := FChDeg;
      ChMin    := FChMin;
      ChSec    := FChSec;
      DecNoSec := FDecNoSec;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.SetModelFile(const ModelFileName: string; const ModelIndex: integer;
  const PlainUnits: TPlainUnits; const UseReversed: boolean; const GridFilesPath: string);
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  FillChar(FModelDescription, SizeOf(TModelDescription), 0);

  FModelFileName:= '';
  FPlainUnits   := PlainUnits;
  FUseReversed  := UseReversed;
  FGridFilesPath:= '';
  FModelIsLoaded:= False;

  if InitTrGEModel(PWideChar(WideString(ModelFileName)), PWideChar(WideString(GridFilesPath)), ModelIndex, FPlainUnits,
    UseReversed, GEEntLangWords, ErrNo, lpErrTxt, nMaxCount) then
    begin
      if GetTrGEModelDescription(FModelDescription) then
        begin
          FModelIsLoaded   := True;
          FModelFileName   := ModelFileName;
          FGridFilesPath   := GridFilesPath;
          FIsTimeDependent := FModelDescription.IsTimeDependent;
          FIsUseTimeEpochT0:= FModelDescription.IsUseTimeEpochT0;

{$IFNDEF UNICODE}
          ChangeCharsOfModelDescription(FModelDescription);
{$ENDIF}
        end;
    end
  else
    begin
      if ErrNo > 0 then
        ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.SetEpochT0(const Value: extended);
begin
  SetTrGEEpochT0(Value);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.GetEpochT0: extended;
begin
  if not GetTrGEEpochT0(Result) then
    Result:= 0.0;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.SetEpochT(const Value: extended);
begin
  SetTrGEEpochT(Value);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.GetEpochT: extended;
begin
  if not GetTrGEEpochT(Result) then
    Result:= 0.0;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.SetGEDrawOptions(const Value: TGEDrawOptions);
begin
  with FGEDrawOptions do
    begin
      StrLCopy(PlgLabelsHtmlColor, Value.PlgLabelsHtmlColor, MAX_LEN_HTML_COLOR);
      StrLCopy(PlgLinesHtmlColor, Value.PlgLinesHtmlColor, MAX_LEN_HTML_COLOR);
      PlgDrawLabels   := Value.PlgDrawLabels;
      PlgAddAreaToName:= Value.PlgAddAreaToName;
      StrLCopy(PathLabelsHtmlColor, Value.PathLabelsHtmlColor, MAX_LEN_HTML_COLOR);
      PathDrawLabels     := Value.PathDrawLabels;
      PathAddLengthToName:= Value.PathAddLengthToName;
      PathIdxSymbIcon    := Value.PathIdxSymbIcon;
      StrLCopy(PtsLabelsHtmlColor, Value.PtsLabelsHtmlColor, MAX_LEN_HTML_COLOR);
      AddDescrToName    := Value.AddDescrToName;
      LabelsDistToViewKm:= Value.LabelsDistToViewKm;
      StrLCopy(FoldPlg, Value.FoldPlg, Length(Value.FoldPlg) + 1);
      StrLCopy(FoldPath, Value.FoldPath, Length(Value.FoldPath) + 1);
      StrLCopy(FoldGrpPts, Value.FoldGrpPts, Length(Value.FoldGrpPts) + 1);
      CorPhi:= Value.CorPhi;
      CorLam:= Value.CorLam;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.AddEntity(const GEEntityProperties: TGEEntityProperties);
begin
  AddGEEntity(GEEntityProperties);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.SetEntity(const EntIndex: integer; const GEEntityProperties: TGEEntityProperties);
begin
  SetGEEntity(EntIndex, GEEntityProperties);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.DeleteEntity(const EntIndex: integer);
begin
  DeleteGEEntity(EntIndex);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.GetEntity(const EntIndex: integer; var GEEntityProperties: TGEEntityProperties): boolean;
begin
  Result:= GetGEEntity(EntIndex, GEEntityProperties);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.ClearEntities;
begin
  ClearGEEntities;
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.GetCountEntites: integer;
begin
  Result:= GetCountGEEntites;
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.AddPointToEntity(const EntIndex: integer; const PntName: string;
  const EorPhi, NorLam: extended);
begin
  AddPointToGEEntity(EntIndex, PWideChar(WideString(PntName)), EorPhi, NorLam);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.SetPointToEntity(const EntIndex, PntIndex: integer; const PntName: string;
  const EorPhi, NorLam: extended);
begin
  SetPointToGEEntity(EntIndex, PntIndex, PWideChar(WideString(PntName)), EorPhi, NorLam);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.GetPointFromEntity(const EntIndex, PntIndex: integer; var PntName: string;
  var EorPhi, NorLam: extended): boolean;
var
  lpPntName       : PWideChar;
  nMaxPntNameCount: integer;
begin
  Result:= GetPointFromGEEntity(EntIndex, PntIndex, lpPntName, nMaxPntNameCount, EorPhi, NorLam);
  SetString(PntName, lpPntName, nMaxPntNameCount);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.GetPointFromEntityFormated(const EntIndex, PntIndex: integer;
  var PntName, StrEorPhi, StrNorLam: string; const CoordIsGeog: boolean): boolean;
var
  lpPntName, lpEorPhi, lpNorLam                     : PWideChar;
  nMaxPntNameCount, nMaxEorPhiCount, nMaxNorLamCount: integer;
begin
  Result:= GetPointFromGEEntityFormated(EntIndex, PntIndex, lpPntName, nMaxPntNameCount, lpEorPhi, nMaxEorPhiCount,
    lpNorLam, nMaxNorLamCount, CoordIsGeog, GetAngleAspect, FDecNoNEH);

  SetString(PntName, lpPntName, nMaxPntNameCount);
  SetString(StrEorPhi, lpEorPhi, nMaxEorPhiCount);
  SetString(StrNorLam, lpNorLam, nMaxNorLamCount);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.GetCountPointsOfEntity(const EntIndex: integer): integer;
begin
  Result:= GetCountPointsOfGEEntity(EntIndex);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.DeleteAllPointsOfEntity(const EntIndex: integer);
begin
  DeleteAllPointsOfGEEntity(EntIndex);
end;

//----------------------------------------------------------------------------------------------------------------------
procedure TTransformEntitiesGE.CalculateAreaAndLengthToEntity(const EntIndex: integer);
begin
  CalculateAreaAndLengthToGEEntity(EntIndex, FAreaUnits, FDecNoArea, FLengthUnits, FDecNoLength);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.LoadEntitiesFromFile(const FileName: string): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  Result:= LoadGEEntitiesFromFile(PWideChar(WideString(FileName)), FInvertAxes12, GetAngleAspect, FChCsv, ErrNo,
    lpErrTxt, nMaxCount);
  if not Result then
    begin
      if ErrNo > 0 then
        ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.SaveEntitiesToFile(const FileName: string; const EncodingType: TEncodingType): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  //Possible extension of file: csv, kml or kmz.
  Result:= SaveGEEntitiesToFile(PWideChar(WideString(FileName)), PWideChar(WideString(FProgName)),
    PWideChar(WideString(FProgVer)), PWideChar(WideString(FWebPage)), PWideChar(WideString(FPlaceName)), FInvertAxes12,
    GetAngleAspect, FChCsv, FGEDrawOptions, GEKmlLangWords, EncodingType, ErrNo, lpErrTxt, nMaxCount);
  if not Result then
    begin
      if ErrNo > 0 then
        ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.DrawEntitiesToGoogleEarth(const TempFileName: string = ''): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  //Google Earth must be installed
  Result:= DrawGEEntitiesToGoogleEarth(PWideChar(WideString(FProgName)), PWideChar(WideString(FProgVer)),
    PWideChar(WideString(FWebPage)), PWideChar(WideString(FPlaceName)), PWideChar(WideString(TempFileName)),
    FGEDrawOptions, GEKmlLangWords, ErrNo, lpErrTxt, nMaxCount);

  if not TempFilesIsUsed then
    TempFilesIsUsed:= Result;

  if ErrNo > 0 then
    ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.SaveZoomToFile(const FileName, FolderName: string;
  const PhiMin, LamMin, PhiMax, LamMax: extended): boolean;
begin
  //Only kml extension of file.
  Result:= SaveGEZoomToFile(PWideChar(WideString(FileName)), PWideChar(WideString(FolderName)), PhiMin, LamMin,
    PhiMax, LamMax);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.DrawZoomToGoogleEarth(const PhiMin, LamMin, PhiMax, LamMax: extended;
  const FolderName: string = ''; const TempFileName: string = ''): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  //Google Earth must be installed
  Result:= DrawGEZoomToGoogleEarth(PWideChar(WideString(TempFileName)), PWideChar(WideString(FolderName)), PhiMin,
    LamMin, PhiMax, LamMax, ErrNo, lpErrTxt, nMaxCount);

  if not TempFilesIsUsed then
    TempFilesIsUsed:= Result;

  if ErrNo > 0 then
    ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
end;

//----------------------------------------------------------------------------------------------------------------------
function TTransformEntitiesGE.TransformPointsOfAllEntities(ProgressProc: TProgressProc): boolean;
var
  ErrNo, nMaxCount: integer;
  lpErrTxt        : PWideChar;
begin
  //It is recommended as the points to be in projected coordinate system (N,E).
  Result:= TransformPointsOfAllGEEntities(ProgressProc, PWideChar(WideString(StrCooOpForErr)), ErrNo, lpErrTxt,
    nMaxCount);
  if not Result then
    begin
      if ErrNo > 0 then
        ShowMessageError(ErrNo, lpErrTxt, nMaxCount);
    end;
end;

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------

initialization

//For other language change these texts:
//- texts used to define entities
with GEEntLangWords do
  begin
    StrWCopy(StrEntMyPlace, PChar('My Place'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrEntPolygonNoName, PChar('Polygon - No name'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrEntPathNoName, PChar('Path - No name'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrEntGrpPtsNoName, PChar('Group points - No name'), MAX_LEN_ENTITY_INFO);
  end;
//- texts used at drawing in Google Earth
with GEKmlLangWords do
  begin
    StrWCopy(StrKmlPlgNumber, PChar('Number'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlPathStart, PChar('Start'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlPathEnd, PChar('End'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlGrpPtsName, PChar('Group name'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlDescription, PChar('Description'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlArea, PChar('Area'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlPerimeter, PChar('Perimeter'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlLength, PChar('Length'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlInfoAd1, PChar('Additional information'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlInfoAd2, PChar('Additional information'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlInfoAd3, PChar('Additional information'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlInfoAd4, PChar('Additional information'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlInfoAd5, PChar('Additional information'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlInfoAd6, PChar('Additional information'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlInformation, PChar('Information'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlAddress, PChar('Address'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlDestination, PChar('Destination'), MAX_LEN_ENTITY_INFO);
    StrWCopy(StrKmlUsage, PChar('Usage'), MAX_LEN_ENTITY_INFO);
  end;
//- texts used for folder names in Google Earth
StrFoldPolygons:= 'Polygons';
StrFoldPaths   := 'Paths';
StrFoldGroupPts:= 'Points Groups';
//- text that is used to display errors about coordinate operations
StrCooOpForErr:= 'Coordinate Operation';

end.
